@@ -1,5 +1,6 @@
 package com.unciv.logic.city
 
+import com.unciv.GUI
 import com.unciv.UncivGame
 import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.automation.Automation
@@ -21,12 +22,12 @@ import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
 import com.unciv.models.translations.tr
-import com.unciv.ui.civilopedia.CivilopediaCategories
-import com.unciv.ui.civilopedia.FormattedLine
-import com.unciv.ui.utils.Fonts
-import com.unciv.ui.utils.extensions.withItem
-import com.unciv.ui.utils.extensions.withoutItem
-import com.unciv.ui.worldscreen.unit.actions.UnitActions
+import com.unciv.ui.screens.civilopediascreen.CivilopediaCategories
+import com.unciv.ui.screens.civilopediascreen.FormattedLine
+import com.unciv.ui.components.Fonts
+import com.unciv.ui.components.extensions.withItem
+import com.unciv.ui.components.extensions.withoutItem
+import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
 import kotlin.math.ceil
 import kotlin.math.roundToInt
 
@@ -673,8 +674,7 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         city.cityStats.update()
         city.civ.cache.updateCivResources()
         // If bought the worldscreen will not have been marked to update, and the new improvement won't show until later...
-        if (UncivGame.isCurrentInitialized() && UncivGame.Current.worldScreen != null)
-            UncivGame.Current.worldScreen!!.shouldUpdate = true
+        GUI.setUpdateWorldOnNextRender()
     }
 
     /** Support for [UniqueType.CreatesOneImprovement]:
