@@ -21,7 +21,7 @@ import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.stats.Stat
-import com.unciv.ui.victoryscreen.RankingType
+import com.unciv.ui.screens.victoryscreen.RankingType
 import java.util.*
 import kotlin.math.min
 import kotlin.math.pow
