@@ -19,8 +19,8 @@ import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
 import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.hasPlaceholderParameters
-import com.unciv.ui.utils.MayaCalendar
-import com.unciv.ui.worldscreen.unit.actions.UnitActionsUpgrade
+import com.unciv.ui.components.MayaCalendar
+import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsUpgrade
 import kotlin.math.roundToInt
 import kotlin.random.Random
 
@@ -53,7 +53,9 @@ object UniqueTriggerActivation {
             UniqueType.OneTimeFreeUnit -> {
                 val unitName = unique.params[0]
                 val unit = ruleSet.units[unitName]
-                if (chosenCity == null || unit == null || (unit.hasUnique(UniqueType.FoundCity) && civInfo.isOneCityChallenger()))
+                if (chosenCity == null
+                        || unit == null
+                        || unit.hasUnique(UniqueType.FoundCity) && civInfo.isOneCityChallenger())
                     return false
 
                 val placedUnit = civInfo.units.addUnit(unitName, chosenCity) ?: return false
@@ -129,13 +131,11 @@ object UniqueTriggerActivation {
                 if (civInfo.isSpectator()) return false
                 civInfo.policies.freePolicies++
 
-
                 val notificationText = getNotificationText(notification, triggerNotificationText,
                     "You may choose a free Policy")
                     ?: return true
 
                 civInfo.addNotification(notificationText, NotificationCategory.General, NotificationIcon.Culture)
-
                 return true
             }
             UniqueType.OneTimeAmountFreePolicies -> {
@@ -148,18 +148,31 @@ object UniqueTriggerActivation {
                     ?: return true
 
                 civInfo.addNotification(notificationText, NotificationCategory.General, NotificationIcon.Culture)
-
                 return true
             }
-            UniqueType.OneTimeEnterGoldenAge -> {
-                civInfo.goldenAges.enterGoldenAge()
+            UniqueType.OneTimeAdoptPolicy -> {
+                val policyName = unique.params[0]
+                if (civInfo.policies.isAdopted(policyName)) return false
+                val policy = civInfo.gameInfo.ruleset.policies[policyName] ?: return false
+                civInfo.policies.freePolicies++
+                civInfo.policies.adopt(policy)
+
+                val notificationText = getNotificationText(notification, triggerNotificationText,
+                    "You gain the [$policyName] Policy")
+                    ?: return true
+
+                civInfo.addNotification(notificationText, NotificationCategory.General, NotificationIcon.Culture)
+                return true
+            }
+            UniqueType.OneTimeEnterGoldenAge, UniqueType.OneTimeEnterGoldenAgeTurns -> {
+                if (unique.type == UniqueType.OneTimeEnterGoldenAgeTurns) civInfo.goldenAges.enterGoldenAge(unique.params[0].toInt())
+                else civInfo.goldenAges.enterGoldenAge()
 
                 val notificationText = getNotificationText(notification, triggerNotificationText,
                     "You enter a Golden Age")
                     ?: return true
 
                 civInfo.addNotification(notificationText, NotificationCategory.General, NotificationIcon.Happiness)
-
                 return true
             }
 
@@ -278,6 +291,18 @@ object UniqueTriggerActivation {
 
                 return true
             }
+            UniqueType.OneTimeDiscoverTech -> {
+                val techName = unique.params[0]
+                if (civInfo.tech.isResearched(techName)) return false
+                civInfo.tech.addTechnology(techName)
+
+                val notificationText = getNotificationText(notification, triggerNotificationText,
+                    "You have discovered the secrets of [$techName]")
+                    ?: return true
+
+                civInfo.addNotification(notificationText, NotificationCategory.General, NotificationIcon.Science)
+                return true
+            }
 
             UniqueType.StrategicResourcesIncrease -> {
                 civInfo.cache.updateCivResources()
@@ -624,7 +649,7 @@ object UniqueTriggerActivation {
                     unit.civ.addNotification(notification, unit.getTile().position, NotificationCategory.Units, unit.name)
                 return true
             }
-            else -> return false
+            else -> return triggerCivwideUnique(unique, civInfo = unit.civ, tile=unit.currentTile)
         }
     }
 }
